-- SERVER

require 'TSerial'
local socket = require 'socket'
clients = {}

function love.load()
	server = socket.tcp()
	server:bind('*',31325)
	server:settimeout(0)
	ip, port = server:getsockname()
	server:listen()
end

function love.update(dt)
	local c = server:accept()
	
	if c then
		c:settimeout(0)
		table.insert(clients, c)
		c:send('%C01'..'Welcome to Whisper! (ensayia.no-ip.org:'..port..')'..'\n')
		c:send('%C01'..'\n')
		c:send('%C01'..'DISCLAIMER: Whisper is NOT secure at this point in time, all data\n')
		c:send('%C01'..'is sent without encryption and there is no authentication anywhere.\n')
		c:send('%C01'..'Do not discuss sensitive information using this program.\n')
		c:send('%C01'..'-------------------------------------------------------------------\n')
	end
	
	for k,v in ipairs(clients) do
		local msg = v:receive('*l')
		if msg then
			local code = string.sub(msg, 1, 4)
			local newmsg = string.sub (msg, 5)
			if code == '%S01' then
				for l, w in ipairs(clients) do
					w:send('%C01'..clients[v]..': '..newmsg..'\n')
				end
			elseif code == '%S02' then
				if clients[v] then
					local old = clients[v]
					clients[v] = newmsg
					for l, w in ipairs(clients) do
						w:send('%C01'..old..' is now known as '..clients[v]..'\n')
						local clist = clientList()
						for l, w in ipairs(clients) do
						w:send('%C02'..clist..'\n')
				end
					end
				else
					clients[v] = newmsg
					local clist = clientList()
					for l, w in ipairs(clients) do
						w:send('%C01'..newmsg..' has joined the server.\n')
						w:send('%C02'..clist..'\n')
					end	
				end
			elseif code == '%S03' then
				-- stuff
			elseif code == '%S04' then
				for l, w in ipairs(clients) do
					w:send('%C01'..clients[v]..' has left the server.\n')
				end
				table.remove(clients, k)
				v:close()
				local clist = clientList()
				for l, w in ipairs(clients) do
					w:send('%C02'..clist..'\n')
				end
			else
				table.remove(clients, k)
				v:close()
			end
		end
	end
end

function love.draw()
	love.graphics.print('Clients: '..#clients, 10, 10)
end

function love.quit()
	for k,v in ipairs(clients) do
		v:send('%C01SERVER CLOSED'..'\n')
		for k,v in ipairs(clients) do
			v:close()
		end
	end
end

function clientList()
	local t = {}
	for k, v in ipairs(clients) do
		table.insert(t, clients[v])
	end
	local clist = TSerial.pack(t)
	return clist
end

-- Command Prefixes
-- %S01 chat message
-- %S02 nick send
-- %S03 pass send
-- %S04 disconnect